/** accumulate_areas.c  accumulate_vertex_areas, accumulate_xsect_areas
Wm. Randolph Franklin
Last modified: by Wm Randolph Franklin (wrf) on mab.ecse.rpi.edu at Sun Nov 24 20:19:38 1991
	%W%	%G%	*/

/* Accumulate the areas of the output polygons due to the input vertices. */

#include "overlay.h"

/* Calc unscaled squared length of edge from dx and dy */

#define CALCDD(dx,dy)  ( rx*ry*(SQ((float)(dx))+SQ((float)(dy))) )


accumulate_vertex_areas(mapno, area_table)
    int     mapno;
    HASH_RECORD_T *area_table;
{
    int     ie, pol1, pol2;
    long int outpol;
    /*
     * Keep dx, dy float since otherwise they would have to be coerced to float
     * whenever used.
     */
    float   dx, dy;
    float   a, a0, a1, a2, dd;
    VERTEX_T *v1, *v2;
    EDGE_T *this_edge;
    int     ne = n_edges[mapno];
    long int badpol;

    /* Trace one output polygon, e.g., badpol = OUTPUT_POLYGON(28,28); */

    for (ie = 0; ie < ne; ie++) {

	this_edge = &edges[mapno][ie];
	v1 = &vertices[mapno][this_edge->v1];
	v2 = &vertices[mapno][this_edge->v2];

	dx = this_edge->dx = v2->x - v1->x;
	dy = this_edge->dy = v2->y - v1->y;
	dd = CALCDD(dx, dy);

	if (dd == 0.0) {
	    printf("Error in accumulate_areas: Zero length edge, map #%d, edge #%d, lpol= %d, rpol= %d\n",
		   mapno, ie, this_edge->pl, this_edge->pr);
	    continue;
	}
	a1 = (v1->x * dx + v1->y * dy) * (float) (v1->x * dy - v1->y * dx) / dd;
	a2 = (v2->x * dx + v2->y * dy) * (float) (v2->x * dy - v2->y * dx) / dd;

	/*
	 * The size of the following code could be reduced by using arrays for the 2
	 * ends of each edge, etc.  However the structures would be bigger and the
	 * code would be slower.
	 */

	/* 1st end of edge */
	a0 = a1;
	pol2 = v1->p;

	/* left side of 1st end */
	pol1 = this_edge->pl;
	a = -a0;
	outpol = ((mapno == 0) ? OUTPUT_POLYGON(pol1, pol2) : OUTPUT_POLYGON(pol2, pol1));

	/**	if (outpol==badpol)  printf("Adding area %f to badpol, map=%d, ie=%d, 1l, edge verts= %d %d, edge pols= %d %d\n", a, mapno, ie, this_edge->v1, this_edge->v2, this_edge->pl, this_edge->pr); */

	find_hash_key_add_value(area_table, (HASH_KEY_T) outpol, a);

	/* right side of 1st end */
	pol1 = this_edge->pr;
	a = a0;
	outpol = ((mapno == 0) ? OUTPUT_POLYGON(pol1, pol2) : OUTPUT_POLYGON(pol2, pol1));
	/**	if (outpol==badpol)  printf("Adding area %f to badpol, map=%d, ie=%d, 1r, edge verts= %d %d, edge pols= %d %d\n", a, mapno, ie, this_edge->v1, this_edge->v2, this_edge->pl, this_edge->pr); */
	find_hash_key_add_value(area_table, (HASH_KEY_T) outpol, a);


	/* 2nd end of edge */
	a0 = a2;
	pol2 = v2->p;

	/* left side of 2nd end (looking from 2nd end) */
	pol1 = this_edge->pr;
	a = -a0;
	outpol = ((mapno == 0) ? OUTPUT_POLYGON(pol1, pol2) : OUTPUT_POLYGON(pol2, pol1));
	/**	if (outpol==badpol)  printf("Adding area %f to badpol, map=%d, ie=%d, 2l, edge verts= %d %d, edge pols= %d %d\n", a, mapno, ie, this_edge->v1, this_edge->v2, this_edge->pl, this_edge->pr); */
	find_hash_key_add_value(area_table, (HASH_KEY_T) outpol, a);

	/* right side of 2nd end */
	pol1 = this_edge->pl;
	a = a0;
	outpol = ((mapno == 0) ? OUTPUT_POLYGON(pol1, pol2) : OUTPUT_POLYGON(pol2, pol1));
	/**	if (outpol==badpol)  printf("Adding area %f to badpol, map=%d, ie=%d, 2r, edge verts= %d %d, edge pols= %d %d\n", a, mapno, ie, this_edge->v1, this_edge->v2, this_edge->pl, this_edge->pr); */
	find_hash_key_add_value(area_table, (HASH_KEY_T) outpol, a);
    }
}


accumulate_xsect_areas(xsect, nxsect, area_table)
    XSECT_T xsect[];
    int     nxsect;
    HASH_RECORD_T *area_table;
{
    int     i;
    long int outpol;
    float   a, a1, a2, dd1, dd2;
    int     p1l, p1r, p2l, p2r;
    int     cas;		/* to help debugging */
    EDGE_T *edge1, *edge2;
    long int badpol;

    /* badpol= -999999; */

    for (i = 0; i < nxsect; i++) {

	edge1 = &edges[0][xsect[i].e1];
	edge2 = &edges[1][xsect[i].e2];

	dd1 = CALCDD(edge1->dx, edge1->dy);
	dd2 = CALCDD(edge2->dx, edge2->dy);

	a1 = (xsect[i].x * (float) edge1->dx + xsect[i].y * (float) edge1->dy) *
	    (xsect[i].x * (float) edge1->dy - xsect[i].y * (float) edge1->dx) / dd1;
	a2 = (xsect[i].x * (float) edge2->dx + xsect[i].y * (float) edge2->dy) *
	    (xsect[i].x * (float) edge2->dy - xsect[i].y * (float) edge2->dx) / dd2;

	p1l = edge1->pl;
	p1r = edge1->pr;
	p2l = edge2->pl;
	p2r = edge2->pr;

	a = a1 - a2;
	if ((edge1->dx * (float) edge2->dy) < (edge1->dy * (float) edge2->dx))
	    a = -a;

	outpol = OUTPUT_POLYGON(p1l, p2l);
	cas = 11;
	/**	if (outpol==badpol) printf("Adding area %f to badpol, ix=%d, edges=%d %d, case ll\n", a, i, xsect[i].e1, xsect[i].e2); */
	find_hash_key_add_value(area_table, (HASH_KEY_T) outpol, a);

	outpol = OUTPUT_POLYGON(p1r, p2r);
	cas = 22;
	/**	if (outpol==badpol) printf("Adding area %f to badpol, ix=%d, edges=%d %d, case rr\n", a, i, xsect[i].e1, xsect[i].e2); */
	find_hash_key_add_value(area_table, (HASH_KEY_T) outpol, a);

	a = -a;

	outpol = OUTPUT_POLYGON(p1l, p2r);
	cas = 12;
	/**	if (outpol==badpol) printf("Adding area %f to badpol, ix=%d, edges=%d %d, case lr\n", a, i, xsect[i].e1, xsect[i].e2); */
	find_hash_key_add_value(area_table, (HASH_KEY_T) outpol, a);

	outpol = OUTPUT_POLYGON(p1r, p2l);
	cas = 21;
	/**	if (outpol==badpol) printf("Adding area %f to badpol, ix=%d, edges=%d %d, case rr\n", a, i, xsect[i].e1, xsect[i].e2); */
	find_hash_key_add_value(area_table, (HASH_KEY_T) outpol, a);


    }
}


total_the_areas(area_table)
    HASH_RECORD_T *area_table;
{
    HASH_RECORD_T *(inarea[2]), *(pinarea[2]), *poutarea;	/* p=packed */
    int     pol1, pol2, noutpol, i, n, mapno;
    HASH_KEY_T outpol;
    float   a;

    noutpol = get_hash_n_records(area_table);
    printf("\n# output polygons=%d\n", noutpol);
    poutarea = pack_hash_table(area_table);

    for (mapno = 0; mapno < 2; mapno++)
	inarea[mapno] = create_hash_table(n_pols[mapno] * 2);	/* 50% occupancy for
								 * hash table */

    for (i = 0; i < noutpol; i++) {
	outpol = (poutarea + i)->k;
	pol1 = INPUT_POLYGON_1(outpol);
	pol2 = INPUT_POLYGON_2(outpol);

	a = (poutarea + i)->v;
	find_hash_key_add_value(inarea[0], (HASH_KEY_T) pol1, a);
	find_hash_key_add_value(inarea[1], (HASH_KEY_T) pol2, a);
    }

    for (mapno = 0; mapno < 2; mapno++) {
	printf("\nAreas of input polygons in map %d, from summing over the output polygons\n", mapno);
	n = get_hash_n_records(inarea[mapno]);
	if (n != n_pols[mapno])
	    printf("Calcareas found %d polygons in map %d, while total the areas finds %d.\n", n, mapno, n_pols[mapno]);

	pinarea[mapno] = pack_hash_table(inarea[mapno]);

	for (i = 0; i < n; i++)
	    printf("%5d: %9.5g\n", (pinarea[mapno] + i)->k, 0.5 * (pinarea[mapno] + i)->v);
    }
}
