/** grid.c  Put the edges of map into a grid

Wm. Randolph Franklin
Last modified: Wed Nov 20 16:29:52 EST 1991
	%W%	%G%	*/
/*
 * The grid data structure is a square array of pointers to arrays, which are
 * realloced and grown when necessary. For space each cell is initially left a null
 * pointer until the first edge is inserted.
 */

/** Array elements for each grid cell:
  0: Size of this array,
  1: Number of elements used so far,
  2: Loc of 1st edge of map #1 in this array (not set till map #0 all entered),
  CELL_HEADER_LEN=3,4,...a[2]-1:  edges of map #0
  a[2],...a[1]-1: edges of map #1
*/

#include "overlay.h"
#include <stdio.h>

int     no_cell_increases;
int     initialsize = 6;	/* Initial array size per cell */
make_grid()
{
    int     i, j;

    no_cell_increases = 0;
    if (grid_size <= 0)
	myerror("Grid size must be positive.");

    /* The 0.9999 is to prevent any points at the right side */
    cell_scale = (float) grid_size / (float) INTERNAL_RANGE *0.9999;

    grid = (EDGE_NO_T ***) malloc2((unsigned) (grid_size * sizeof(EDGE_NO_T **)),
				   "malloc failure trying to alloc grid-1");

    for (i = 0; i < grid_size; i++) {
	grid[i] =
	    (EDGE_NO_T **) malloc2((unsigned) (grid_size * sizeof(EDGE_NO_T *)),
				   "malloc failure trying to alloc grid-2");
	for (j = 0; j < grid_size; j++) {
	    grid[i][j] = NULL;
	}
    }

    /*
     * Allocate the array to store which polygon each cell is completely in, if any.
     */

    FOR2 {
	polygon_containing_cell[im] =
	    (POLYGON_ID_T **)
	    malloc2((unsigned) (grid_size * sizeof(POLYGON_ID_T *)),
		    "malloc failure trying to alloc polygon_containing_cell-1");

	for (i = 0; i < grid_size; i++) {
	    polygon_containing_cell[im][i] =
		(POLYGON_ID_T *)
		malloc2((unsigned) (grid_size * sizeof(POLYGON_ID_T)),
			"malloc failure trying to alloc polygon_containing_cell-2");

	    for (j = 0; j < grid_size; j++) {
		polygon_containing_cell[im][i][j] = DONTKNOW;
	    }
	}
    }


}



/*
 * Add the edges of this map to the grid.  If this is map #0, then store in each
 * cell the loc of the next free edge slot.
 */


add_map_to_grid(v, e, ne, mapno)
    VERTEX_T v[];
    EDGE_T  e[];
    int     ne, mapno;

{
    int     ix1, iy1, ix2, iy2, temp;
    int     i, j, pl, pr;
    long int v1, v2;
    float   x1, y1, x2, y2;
    EDGE_NO_T iedge, *arr;

    for (iedge = 0; iedge < ne; iedge++) {
	GET_EDGE_COORDS(iedge);

	ix1 = CELL(x1);
	iy1 = CELL(y1);
	ix2 = CELL(x2);
	iy2 = CELL(y2);

	/* Be lazy for now.  Put a box around the edge */

	if (ix1 > ix2)
	    SWAP(ix1, ix2);
	if (iy1 > iy2)
	    SWAP(iy1, iy2);

	if (debug_add_map_to_grid && ix1 < ix2 && iy1 < iy2)
	    printf("Edge %d llc cells %d %d, urc %d %d\n", iedge, ix1, iy1, ix2, iy2);

	for (i = ix1; i <= ix2; i++)
	    for (j = iy1; j <= iy2; j++)
		add_edge_to_cell(i, j, iedge);
    }

    /*
     * If this is map #0, record in each cell the loc of next free slot, so we'll
     * know where map #1 starts.
     */

    if (mapno == 0)
	for (i = 0; i < grid_size; i++)
	    for (j = 0; j < grid_size; j++) {
		arr = grid[i][j];
		if (arr != NULL)
		    arr[2] = arr[1];
	    }

}



add_edge_to_cell(i, j, edge)
    EDGE_NO_T edge;
    int     i, j;
{
    EDGE_NO_T *arr;
    int     m, m2, n;
    arr = grid[i][j];
    if (arr == NULL) {		/* No edges in this cell so far */
	arr = grid[i][j] =
	    (EDGE_NO_T *) malloc2((unsigned) (initialsize * sizeof(EDGE_NO_T)),
				  "malloc failure trying to alloc a grid cell");
	m = arr[0] = initialsize;
	n = arr[1] = arr[2] = CELL_HEADER_LEN;
    } else {
	m = arr[0];
	n = arr[1];
    }
    if (n == m) {		/* alloc more space */
	m2 = m * 2;
	grid[i][j] = arr =
	    (EDGE_NO_T *) realloc((char *) arr, (unsigned) (m2 * sizeof(EDGE_NO_T)));
	if (arr == NULL)
	    myerror("Realloc failure trying to grow grid cell");
	arr[0] = m2;
	no_cell_increases++;
	if (debug_add_edge_to_cell)
	    printf("Increasing size of cell %d,%d to %d\n", i, j, m2);
    };
    arr[n++] = edge;
    arr[1] = n;
}



void
print_grid()
{
    int     i, j;
    EDGE_NO_T k, *arr;
    for (i = 0; i < grid_size; i++)
	for (j = 0; j < grid_size; j++) {
	    arr = grid[i][j];
	    printf("[%3d,%3d]:", i, j);
	    if (arr != NULL)
		for (k = 0; k < arr[1]; k++)
		    printf(" %3d", arr[k]);
	    printf("\n");
	};
}



void
print_grid_histogram()
{
    int     hist[MAX_GRID_HISTOGRAM];
    int     i, j, k, n;
    EDGE_NO_T *arr;

    for (i = 0; i < MAX_GRID_HISTOGRAM; hist[i++] = 0);
    for (i = 0; i < grid_size; i++)
	for (j = 0; j < grid_size; j++) {
	    arr = grid[i][j];
	    if (arr == NULL)
		n = 0;
	    else
		n = arr[1] - CELL_HEADER_LEN;	/* Number of edges in this cell,
						 * excl header info */
	    if (n >= MAX_GRID_HISTOGRAM) {
		printf("Warning: Cell (%d,%d) has %d edges: ", i, j, n);
		for (k = 0; k < n; k++)
		    printf(" %3d", arr[k + CELL_HEADER_LEN]);
		printf("\n");
		n = MAX_GRID_HISTOGRAM - 1;
	    }
	    hist[n]++;
	};

    /* Find highest non-zero element */
    for (j = MAX_GRID_HISTOGRAM - 1; j >= 0; j--)
	if (hist[j] != 0)
	    break;

    printf("\n= Histogram of number of cells with n edges, for 0<=n<=%d:\n", j);
    for (i = 0; i <= j; i++)
	printf(" %d", hist[i]);
    printf("\n");

    printf("Number of times grid cells had to be realloced with more space= %d\n",
	   no_cell_increases);
}
