/** Hash.c   Hash table routines

There are 2 sets of hashing routines.

1. Store partial polygon areas.  The hash record contains  the key and
the float area.  There is an op for adding to an existing area.

2. Store polygon ids.  The has record contains only the key.  There is
an op for returning  the has location  and automaticaaly assigning one
if it doesn't exist.

FOR THE FIRST TYPE:

Keys are long ints, values are floats

create_hash_table(m) Return a new table of m  elements. m must include
extra space for hashing efficiency.

The  hash function is quite  simple:  location=(key)%(table size).  If
this causes problems in the future, it can be changed.

The table is actually allocated  HASH_TABLE_DISP =  2  entries larger,
with the 0th entry key holding the  max number of  entries and the 1st
the actual number.

The key must be  a positive int.   0 keys are ignored for convenience,
since they arise in calc_areas, and negative keys cause an error.

find_hash_key(table,key) Return the associated value, or 0.

find_hash_key_add_value(table,key,value) If key doesn't  exist, insert
it with value.  If  it does exist,  then add the  value to the current
value.  Return the value.

pack_hash_table(table) Return a pointer  to a compacted table with the
zero entries and the two header entries removed.  Sort the table.

get_hash_n_records(table) Return the current number of records.

Wm. Randolph Franklin

Last modified: Fri Jul  6 16:24:50 EDT 1990

	%W%	%G%	*/

#include "overlay.h"

HASH_RECORD_T *
create_hash_table(m)
    int     m;
{
    HASH_RECORD_T *p;
    if (m <= 0)
	myerror("m must be positive in create_hash_table.");
    p = (HASH_RECORD_T *) calloc((unsigned) (m + HASH_TABLE_DISP), sizeof(HASH_RECORD_T));
    p->k = m;
    (p + 1)->k = 0;
    return (p);
};

float
find_hash_key(table, key)
    HASH_RECORD_T *table;
    HASH_KEY_T key;
{
    int     i, j, m, m8;
    HASH_RECORD_T *top, *bot, *p;
    float   val;

    if (key <= 0)
	if (key < 0)
	    myerror("In find_hash_key_add_value, the hash key must be a positive int.");
	else
	    return 0.0;

    m = table->k;
    m8 = m * 8 / 10;
    i = (key) % m;
    bot = table + HASH_TABLE_DISP;
    top = bot + m;
    p = bot + i;
    val = 0.0;
    for (j = 0; j < m; j++) {
	if (p->k == key) {
	    val = p->v;
	    break;
	};
	if (p->k == 0) {
	    val = 0;
	    break;
	};
	p++;
	if (p >= top)
	    p = bot;
    };
    if (j >= m8) {
	printf("Hash table size = %d, .8 size=%d\n", m, m8);
	myerror("Hash table too full in find_hash_key");
    }
    if (j >= 3)
	printf("In find_hash_key_add_value, key %d took %d probes.\n", key, j);
    return (val);
}


float
find_hash_key_add_value(table, key, value)
    HASH_RECORD_T *table;
    float   value;
    HASH_KEY_T key;
{
    int     i, j, m, m8;
    HASH_RECORD_T *top, *bot, *p;
    float   val;

    if (key <= 0)
	if (key < 0)
	    myerror("In find_hash_key_add_value, the hash key must be a positive int.");
	else
	    return 0.0;

    m = table->k;
    m8 = m * 8 / 10;
    i = (key) % m;
    bot = table + HASH_TABLE_DISP;
    top = bot + m;
    p = bot + i;
    val = 0.0;
    for (j = 0;; j++) {
	if (j >= m8) {
	    printf("Hash table size = %d, .8 size=%d\n", m, m8);
	    myerror("Hash table too full in find_hash_key_add_value");
	}
	if (p->k == key) {
	    val = (p->v += value);
	    break;
	};
	if (p->k == 0) {
	    p->k = key;
	    val = p->v = value;
	    (table + 1)->k += 1;/* # entries in table */
	    break;
	};
	p++;
	if (p >= top)
	    p = bot;
    };
    if (j >= 3)
	printf("In find_hash_key_add_value, key %d took %d probes.\n", key, j);

    return (val);
}


HASH_RECORD_T *
pack_hash_table(table)
    HASH_RECORD_T *table;
{
    int     n;
    HASH_RECORD_T *p, *r, *s, *pbot, *ptop;
    int     hash_compar();

    n = get_hash_n_records(table);
    s = r = (HASH_RECORD_T *) calloc((unsigned) n, sizeof(HASH_RECORD_T));
    pbot = table + HASH_TABLE_DISP;
    ptop = pbot + (table->k);	/* 1 past top */
    for (p = pbot; p != ptop; p++) {
	if (p->k != 0) {
	    s->k = p->k;
	    (s++)->v = p->v;
	};
    };
    if (s != r + n)
	myerror("Wrong number in pack_hash_table");

    /* Sort the table. */

    qsort((char *) r, n, sizeof(HASH_RECORD_T), hash_compar);

    return (r);
}

int
get_hash_n_records(table)
    HASH_RECORD_T *table;
{
    return ((table + 1)->k);
}


int
hash_compar(a, b)
    HASH_RECORD_T *a, *b;
{
    n_hash_compar++;
    return (a->k - b->k);
}
