/** intersect_edges.c  Intersect all the edges

  Wm. Randolph Franklin
Last modified: by Wm Randolph Franklin (wrf) on mab.ecse.rpi.edu at Wed Nov 20 17:16:27 1991
	%W%	%G%	*/

/*
 * Remember that the 1st CELL_HEADER_LEN elts of each grid cell are the number of
 * allocated entries, the number used up to the end of map #0, and the total number
 * of cells used including this header and both maps, (CELL_HEADER_LEN more than the
 * number of edges).
 */

#include "overlay.h"
#include <stdio.h>

void    intersect_one_cell();

static VERTEX_T *v0, *v1;	/* globalize the args */
static EDGE_T *e0, *e1;
static XSECT_T *xsect;
static int nxsect, ig, jg;


intersect_edges(av0, ae0, av1, ae1, axsect, anxsect)
    VERTEX_T *av0, *av1;
    EDGE_T *ae0, *ae1;
    XSECT_T *axsect;
    int    *anxsect;		/* Number of intersections */

{
    /* Copy over args for speed so they don't have to be passed explicitly */
    v0 = av0;
    v1 = av1;
    e0 = ae0;
    e1 = ae1;
    xsect = axsect;

    nxsect = 0;
    n_edge_pairs_tested_for_xsect = 0;
    for (ig = 0; ig < grid_size; ig++)
	for (jg = 0; jg < grid_size; jg++) {
	    intersect_one_cell();
	}
    *anxsect = nxsect;
}


/* intersect the edges in one cell */

void
intersect_one_cell()
{
    int     i, j, ie, je, ne, base0, base1;
    EDGE_NO_T *cell;
    VERT_COORD_T x0, y0, x1, y1, xx0, yy0, xx1, yy1, xsectx, xsecty, t;
    INTERMED_COORD_T se00, se01, se10, se11;
    float   denom, numx, numy, fsectx, fsecty;
    int     icell, jcell;
    EDGE_T *ee0, *ee1;

    cell = grid[ig][jg];
    if (cell == NULL)
	return;
    base0 = CELL_HEADER_LEN;
    base1 = cell[2];
    ne = cell[1];
    if (ne == base0)
	return;			/* No edges in this cell */
    if (base0 == base1 || base1 == ne)
	return;			/* All the edges are one type */

    for (i = base0; i < base1; i++) {
	ie = cell[i];		/* The number of the edge of map 0 being xsected */
	ee0 = &e0[ie];		/* A pointer to this edge's struct */
	x0 = v0[ee0->v1].x;	/* The coords of its endpoints */
	y0 = v0[ee0->v1].y;
	x1 = v0[ee0->v2].x;
	y1 = v0[ee0->v2].y;

	for (j = base1; j < ne; j++) {
	    n_edge_pairs_tested_for_xsect++;
	    je = cell[j];
	    ee1 = &e1[je];
	    xx0 = v1[ee1->v1].x;
	    yy0 = v1[ee1->v1].y;
	    xx1 = v1[ee1->v2].x;
	    yy1 = v1[ee1->v2].y;

	    /* Substitute a point into the equation of an edge */

#define SUBEDGE(x,y,e) ((x)*e->a+(y)*e->b+e->c)

	    se00 = SUBEDGE(xx0, yy0, ee0);
	    se10 = SUBEDGE(xx1, yy1, ee0);
	    se01 = SUBEDGE(x0, y0, ee1);
	    se11 = SUBEDGE(x1, y1, ee1);

	    /*
	     * Move map1 by (eps, eps^2) relative to map 0 (Simulation of
	     * Simplicity)
	     */

	    if (se01 == 0)
		se01 = -ee1->a;
	    if (se01 == 0)
		se01 = -ee1->b;
	    if (se01 == 0)
		continue;	/* Zero length edge */
	    if (se11 == 0)
		se11 = -ee1->a;
	    if (se11 == 0)
		se11 = -ee1->b;
	    if (se11 == 0)
		continue;	/* Zero length edge */
	    if ((se01 > 0 && se11 > 0) || (se01 < 0 && se11 < 0))
		continue;

	    if (se00 == 0)
		se00 = ee0->a;
	    if (se00 == 0)
		se00 = ee0->b;
	    if (se00 == 0)
		continue;	/* Zero length edge */
	    if (se10 == 0)
		se10 = ee0->a;
	    if (se10 == 0)
		se10 = ee0->b;
	    if (se10 == 0)
		continue;	/* Zero length edge */
	    if ((se10 > 0 && se00 > 0) || (se10 < 0 && se00 < 0))
		continue;


	    /*
	     * Check if both edges are the same.  If so, they shouldn't be
	     * intersecting.
	     */

	    if ((x0 == xx0 && y0 == yy0 && x1 == xx1 && y1 == yy1) ||
		(x0 == xx1 && y0 == yy1 && x1 == xx0 && y1 == yy0))
		printf("=ERROR: In intersect_edges, cell %d,%d, edges %d, %d, \
which are identical, have been found to intersect.  Endpoints: (%d,%d), (%d,%d)\n",
		       ig, jg, ie, je, x0, y0, x1, y1);

	    /**		    if (se01 * se11 >= 0.0)
				    continue;
			        if (se00 * se10 >= 0.0)
				    continue;  */

	    denom = (float) (ee0->a) * (float) (ee1->b) - (float) (ee1->a) * (float) (ee0->b);
	    numx = ((float) (ee0->c) * (float) (ee1->b) - (float) (ee1->c) * (float) (ee0->b));

	    /*
	     * converting float to int truncates towards 0, not towards -infinity.
	     * We want a round to the nearest int regardless of sign.
	     */

	    fsectx = -numx / denom;
	    if (fsectx > 0.0)
		xsectx = (VERT_COORD_T) (fsectx + 0.5);
	    else
		xsectx = (VERT_COORD_T) (fsectx - 0.5);

	    numy = ((float) (ee0->a) * (float) (ee1->c) - (float) (ee1->a) * (float) (ee0->c));

	    fsecty = -numy / denom;
	    if (fsecty > 0.0)
		xsecty = (VERT_COORD_T) (fsecty + 0.5);
	    else
		xsecty = (VERT_COORD_T) (fsecty - 0.5);

	    /*
	     * Ensure that the intersection point is not in an adjacent cell because
	     * of roundoff.  The following is not perfect, but is better than
	     * nothing.
	     */

	    t = MIN4(x0, x1, xx0, xx1);
	    if (xsectx < t) {
		printf("Intersection of edges %d and %d has x=%d below xmin=%d.  Rounded up.\n", ie, je, xsectx, t);
		xsectx = t;
	    }
	    t = MAX4(x0, x1, xx0, xx1);
	    if (xsectx > t) {
		printf("Intersection of edges %d and %d has x=%d above xmax=%d.  Rounded down.\n", ie, je, xsectx, t);
		xsectx = t;
	    }
	    t = MIN4(y0, y1, yy0, yy1);
	    if (xsecty < t) {
		printf("Intersection of edges %d and %d has y=%d below ymin=%d.  Rounded up.\n", ie, je, xsecty, t);
		xsecty = t;
	    }
	    t = MAX4(y0, y1, yy0, yy1);
	    if (xsecty > t) {
		printf("Intersection of edges %d and %d has y=%d above ymax=%d.  Rounded down.\n", ie, je, xsecty, t);
		xsecty = t;
	    }
	    xsect[nxsect].x = xsectx;
	    xsect[nxsect].y = xsecty;
	    xsect[nxsect].e1 = ie;
	    xsect[nxsect].e2 = je;
	    icell = CELL(xsectx);
	    jcell = CELL(xsecty);
	    if (icell == ig && jcell == jg) {
		if (debug_print_xsect)
		    printf("Xsect #%d, edges %dx%d, (x,y)=(%d,%d), or (%g,%g), cell (%d,%d)\n",
			   nxsect, ie, je, xsectx, xsecty,
			   UNSCALEX(xsectx), UNSCALEY(xsecty),
			   icell, jcell);
	    } else {		/* Intersection not in this cell */
		n_dupl_xsect++;
		nxsect--;
	    }
	    if (MAX_XSECTS <= nxsect++)
		myerror("Too many intersections.\n");
	}

    }
}
