/** ~/c/overlay/overlay.c
Wm. Randolph Franklin
Last modified: by Wm Randolph Franklin (wrf) on mab.ecse.rpi.edu at Wed Nov 20 17:56:29 1991
	%W%	%G%
*/

#include "overlay.h"
#include "lastmoddate.h"
#include <stdio.h>

CHAIN   chains[2][MAX_CHAINS];
int     n_chains[2];
RANGE_T xmins[2], xmaxs[2], ymins[2], ymaxs[2];

int     nxsect;			/* Number of intersections */
XSECT_T xsect[MAX_XSECTS];

main()
{
    char    map_names[2][MAX_MAP_NAME_LEN];
    float   t_locate[2], timers[10], tottime;
    int     i;
    HASH_RECORD_T *area_table;
    int     area_table_size;
    FLOAT_VERTEX_T *fverts[2];


    (void) read_delta_time();

    printf("Overlay was compiled on %s\n", LASTMODDATE);
    printf("= Compiled constants for this run:\n   MAX_CHAINS= %d\n   MAX_VERTS= %d\n",
	   MAX_CHAINS, MAX_VERTS);
    printf("   MAX_EDGES= %d\n   MAX_PTS_PER_CHAIN= %d\n   MAX_MAP_NAME_LEN= %d\n   MAX_XSECTS= %d\n",
	   MAX_EDGES, MAX_PTS_PER_CHAIN, MAX_MAP_NAME_LEN, MAX_XSECTS);
    printf("Sizes of some structures:\n   VERTEX_T= %d\n   FLOAT_VERTEX_T=%d\n",
	   sizeof(VERTEX_T), sizeof(FLOAT_VERTEX_T));
    printf("   EDGE_T= %d\n   HASH_RECORD_T= %d\n   XSECT_T= %d\n   SLAB= %d\n   CHAIN= %d\n",
	   sizeof(EDGE_T), sizeof(HASH_RECORD_T), sizeof(XSECT_T), sizeof(SLAB), sizeof(CHAIN));

    debug_print_grid = 0;	/* These control printing of debug info */
    debug_add_map_to_grid = 0;
    debug_add_edge_to_cell = 0;
    debug_calc_areas = 0;
    debug_inclusions = 0;
    debug_print_xsect = 0;
    debug_print_chains_vertices = 0;
    n_dupl_xsect = 0;
    n_hash_compar = 0;


    /*
     * Copy constants into variables so they will be available at debugging time
     */

    Max_Chains = MAX_CHAINS;
    Max_Verts = MAX_VERTS;
    Max_Edges = MAX_EDGES;
    Max_Pts_Per_Chain = MAX_PTS_PER_CHAIN;
    Max_Map_Name_Len = MAX_MAP_NAME_LEN;
    Max_Xsects = MAX_XSECTS;
    Max_Grid_Histogram = MAX_GRID_HISTOGRAM;
    Cell_Header_Len = CELL_HEADER_LEN;
    Hash_Table_Disp = HASH_TABLE_DISP;
    Polygon_Id_Min = POLYGON_ID_MIN;
    Polygon_Id_Max = POLYGON_ID_MAX;
    Internal_Min = INTERNAL_MIN;
    Internal_Max = INTERNAL_MAX;
    Internal_Range = INTERNAL_RANGE;


    /* Alloc floating vertex space */

    FOR2
    {
	fverts[im] =
	    (FLOAT_VERTEX_T *) malloc2((unsigned) (MAX_VERTS * sizeof(FLOAT_VERTEX_T)),
				       "Malloc of fverts failed in main.");
    }
    /* Prompt for map names and read the chains in floating coords. */

    FOR2 {
	printf("= Map #%d? ", im);
	scanf("%50s", map_names[im]);
	read_pgraph(map_names[im], fverts[im], &n_verts[im], chains[im],
		    &n_chains[im], &xmins[im], &xmaxs[im], &ymins[im], &ymaxs[im]);
    }

    timers[0] = read_delta_time();


    /* Find common bounds and scales */

    ext_xmax = MAX(xmaxs[0], xmaxs[1]);
    ext_ymax = MAX(ymaxs[0], ymaxs[1]);
    ext_xmin = MIN(xmins[0], xmins[1]);
    ext_ymin = MIN(ymins[0], ymins[1]);

    rx = (((float) INTERNAL_RANGE) / (ext_xmax - ext_xmin));
    ry = (((float) INTERNAL_RANGE) / (ext_ymax - ext_ymin));
    rrx = 1.0 / rx;
    rry = 1.0 / ry;

    deltax = 0.5 * ((INTERNAL_MAX + INTERNAL_MIN) - (ext_xmax + ext_xmin) * rx);
    deltay = 0.5 * ((INTERNAL_MAX + INTERNAL_MIN) - (ext_ymax + ext_ymin) * ry);
    ddeltax = 0.5 * ((ext_xmax + ext_xmin) - (INTERNAL_MAX + INTERNAL_MIN) * rrx);
    ddeltay = 0.5 * ((ext_ymax + ext_ymin) - (INTERNAL_MAX + INTERNAL_MIN) * rry);


    /* Scale the float vers into ints. */

    FOR2    scale_vertices(fverts[im], vertices[im], chains[im], n_chains[im]);

    if (debug_print_chains_vertices) {
	FOR2 {
	    printf("-- Map #%d --\n", im);
	    print_vertices(vertices[im], n_verts[im], fverts[im]);
	    print_chains(chains[im], n_chains[im]);
	}
    };

    /* Free the float vertex space */

    FOR2 {
	free((char *) fverts[im]);
	fverts[im] = NULL;
    }

    timers[1] = read_delta_time();


    /* Split the chains into edges. */

    FOR2    extract_edges(im, chains[im], n_chains[im]);

    timers[2] = read_delta_time();


    /* Calculate input polygon areas and print. */

    FOR2    n_pols[im] = calc_areas(vertices[im], edges[im], n_edges[im]);

    FOR2    printf("= Map #%d: %d vertices, %d edges, %d polygons\n", im, n_verts[im], n_edges[im], n_pols[im]);

    timers[3] = read_delta_time();

    printf("= Gridsize? ");
    scanf("%d", &grid_size);
    (void) make_grid();

    timers[4] = read_delta_time();


    /* Determine the cells each edge passes through and insert into grid. */


    FOR2    add_map_to_grid(vertices[im], edges[im], n_edges[im], im);

    if (debug_print_grid)
	print_grid();
    print_grid_histogram();

    timers[5] = read_delta_time();


    /* Intersect all the edges. */

    intersect_edges(vertices[0], edges[0], vertices[1], edges[1], xsect, &nxsect);
    printf("Number of pairs of edges tested for intersection= %d\n",
	   n_edge_pairs_tested_for_xsect);
    printf("Number of intersections found= %d\n", nxsect);
    printf("Number of duplicate intersections found= %d\n", n_dupl_xsect);

    timers[6] = read_delta_time();

    FOR2 {
	locate_vertices_in_other_map();
	t_locate[im] = read_delta_time();
    }

    if (debug_inclusions) {
	printf("\nPolygons in map 0 that the vertices of map 1 fall in:");
	for (i = 0; i < n_verts[1]; i++) {
	    if ((i % 20) == 0)
		printf("\n");
	    printf(" %3d", vertices[1][i].p);
	}
	printf("\n");
	printf("\nPolygons in map 1 that the vertices of map 0 fall in:");
	for (i = 0; i < n_verts[0]; i++) {
	    if ((i % 20) == 0)
		printf("\n");
	    printf(" %3d", vertices[0][i].p);
	}
	printf("\n");
    };

    /*
     * The hash table size is impossible to predict in advance; it might range from
     * the total number of polygons up to the product of the numbers of the polygons
     * in the 2 maps.  Therefore, guess, and if it overflows, then change it.
     */

    area_table_size = (n_pols[0] + n_pols[1] + n_verts[0] + n_verts[1]) * 5;
    area_table = create_hash_table(area_table_size);

    FOR2    accumulate_vertex_areas(im, area_table);

    accumulate_xsect_areas(xsect, nxsect, area_table);

    timers[7] = read_delta_time();

    print_output_areas(area_table);

    /* Check that the components of each polygon add up correctly */

    if (debug_calc_areas)
	total_the_areas(area_table);

    timers[8] = read_delta_time();

    printf("\n= Execution times:\n\
   Read map:                     %5.2f\n\
   Scale vertices:               %5.2f\n\
   Extract edges:                %5.2f\n\
   Calculate areas:              %5.2f\n\
   Make grid:                    %5.2f\n",
	   timers[0], timers[1], timers[2], timers[3], timers[4]);

    printf("\
   Add map to grid:              %5.2f\n\
   Intersect edges:              %5.2f\n\
   Locate map 0 points in map 1: %5.2f\n\
   Locate map 1 points in map 0: %5.2f\n\
   Accumulate areas:             %5.2f\n\
   Print areas:                  %5.2f\n",
	   timers[5], timers[6], t_locate[0], t_locate[1], timers[7], timers[8]);

    tottime = timers[0] + timers[1] + timers[2] + timers[3] + timers[4] +
	timers[5] + timers[6] + timers[7] + timers[8] +
	t_locate[0] + t_locate[1];
    printf("\nTOTAL TIME=                    %7.2f\n\n", tottime);
}
