/** overlay.h
Wm. Randolph Franklin
Last modified: by Wm Randolph Franklin (wrf) on speed.ecse.rpi.edu at Tue Jan 25 12:44:30 1994
	%W%	%G%

Contents:
    includes,
    constants,
    macros,
    types,
    structs,
    variables,
    function decls.

 */



/* ================ Constants ================ */


#define OK	0
#define IN	1
#define OUT	0
#define INFINITY 32767
#define DONTKNOW (-1)

/* Sized for hydrog file */
#define MAX_CHAINS	20000
#define MAX_VERTS 140000
#define MAX_EDGES 120000
#define MAX_PTS_PER_CHAIN 1000
#define MAX_MAP_NAME_LEN 50
#define MAX_XSECTS 20000		/* Max number of intersections */
#define MAX_GRID_HISTOGRAM 300	/* Size of histogram table for # edges per cell */

#define CELL_HEADER_LEN 3	/* Number ofheader elements in each grid cell. */

#define HASH_TABLE_DISP 2	/* displacement from start of table to 1st data */

#define POLYGON_ID_MIN 0
#define POLYGON_ID_MAX 32767

#define INTERNAL_MIN (-16383)	/* legal internal coords after scaling */
#define INTERNAL_MAX  16383
#define INTERNAL_RANGE (INTERNAL_MAX-INTERNAL_MIN)



/* ================ Macros ================ */

/*
 * Scaling the input coords to (-16383,16383) means that the edge eqn can be stored
 * in long ints w/o division
 */

#define SCALEX(x) ((VERT_COORD_T) ((x)*rx+deltax))
#define SCALEY(y) ((VERT_COORD_T) ((y)*ry+deltay))

#define UNSCALEX(x) ((float)(x)*rrx+ddeltax)
#define UNSCALEY(y) ((float)(y)*rry+ddeltay)

#define GET_EDGE_COORDS(i) \
	v1 = e[i].v1; \
	v2 = e[i].v2; \
	pl = e[i].pl; \
	pr = e[i].pr; \
	x1 = v[v1].x; \
	y1 = v[v1].y; \
	x2 = v[v2].x; \
	y2 = v[v2].y

/* Find the cell number from the internal coordinate */

#define CELL(xy)  (int) ((float)((xy)-INTERNAL_MIN)*cell_scale)

#define SWAP(i,j) {temp=i;i=j;j=temp;}

#define MAX3(i,j,k) ( ((i)<(j)) ? \
		     ( ((j)<(k)) ? (k) : (j) ) : \
		     ( ((i)<(k)) ? (k) : (i) ) )

#define MAX(a,b) ((a)>(b)?(a):(b))
#define MIN(a,b) ((a)<(b)?(a):(b))

/* MAX4 Calls MAX(a,b) too many times, but don't worry now. */

#define MAX4(a,b,c,d) (MAX(MAX(a,b),MAX(c,d)))
#define MIN4(a,b,c,d) (MIN(MIN(a,b),MIN(c,d)))

/*
 * Output polygon number as a function of the 2 input polygons.  Note that + binds
 * tighter than <<.
 */

#define OUTPUT_POLYGON(a,b) ( ((a)<<16) + (b) )

/* Input polygon numbers as a function of the output polygon */

#define INPUT_POLYGON_1(a) ((a)>>16)
#define INPUT_POLYGON_2(a) ((a)&65535)

/* Loop for each map */

#define FOR2 for(im=0;im<2;im++)

/* Square */

#define SQ(x) ((x)*(x))



/* ================ Types ================ */

#define VERT_COORD_T  short int
#define VERT_NO_T long int
#define EDGE_NO_T  int		/* too big for unsigned short. Oh well. */
#define HASH_KEY_T long int
#define POLYGON_ID_T short int
#define RANGE_T float		/* of xmin etc */
#define INTERMED_COORD_T long int	/* intermediate type for coordinate
					 * calculations.  If long int is
					 * insufficient, make VERT_COORD_T and this
					 * double. */



/* ================ Structs ================ */

typedef struct VERTEX_T {
    VERT_COORD_T x, y;
    short int p;		/* Which polygon on the other map this vert is in.
				 * -1: dont know.  */
}       VERTEX_T;

typedef struct FLOAT_VERTEX_T {
    float   x, y;
}       FLOAT_VERTEX_T;		/* Vertices before scaling */


typedef struct EDGE_T {
    VERT_NO_T v1, v2;		/* numbers of verts at ends */
    POLYGON_ID_T pl, pr;	/* the numbers of the polygons to left and right */
    INTERMED_COORD_T a, b, c;	/* Eqn: a*x+b*y+c=0 */
    INTERMED_COORD_T dx, dy;	/* Direction of edge: v2-v1. Stored for efficiency.
				 * Calced in accumulate_vertex_areas and used in
				 * accumulate_xsect_areas. */
}       EDGE_T;

typedef struct HASH_RECORD_T {	/* hash table entry */
    HASH_KEY_T k;
    float   v;
}       HASH_RECORD_T;

typedef struct XSECT_T {
    EDGE_NO_T e1, e2;
    VERT_COORD_T x, y;
}       XSECT_T;



typedef struct SLAB {
    EDGE_NO_T edge;
    struct SLAB *next;
}       SLAB;

typedef struct CHAIN {
    int     base;		/* loc, in fverts and vertices, of 1st vert of this
				 * chain */
    int     num_vert;
    POLYGON_ID_T lpol, rpol;
}       CHAIN;



/*
 * =========================== Copies of constants for debugging convenience ====
 */

int     Max_Chains, Max_Verts, Max_Edges, Max_Pts_Per_Chain, Max_Map_Name_Len;
int     Max_Xsects, Max_Grid_Histogram, Cell_Header_Len, Hash_Table_Disp;
int     Polygon_Id_Min, Polygon_Id_Max, Internal_Min, Internal_Max, Internal_Range;



/* =========================== Variables ============================ */


int     grid_size;		/* Number of grid cells per side. Square array of
				 * arrays of edges. */

int     im;			/* In many loops, the number of the current map. */

EDGE_NO_T ***grid;
POLYGON_ID_T **polygon_containing_cell[2];

float   cell_scale;

int     debug_print_grid, debug_add_map_to_grid, debug_add_edge_to_cell, debug_calc_areas;
int     debug_inclusions, debug_print_chains_vertices, debug_print_xsect;

/* Combined bounds of input maps */

RANGE_T ext_xmin, ext_xmax, ext_ymin, ext_ymax;


/* scaling factors between external and internal coords */

RANGE_T rx, ry, rrx, rry;

/* deltas for scaling */

RANGE_T deltax, deltay, ddeltax, ddeltay;

/* Sizes of input maps */

int     n_edges[2], n_verts[2], n_pols[2];

/* Input maps */

VERTEX_T vertices[2][MAX_VERTS];
EDGE_T  edges[2][MAX_EDGES];

/* Counters */

int     n_edge_pairs_tested_for_xsect;
int     n_dupl_xsect;
int     n_hash_compar;		/* # times hash_compare called when sorting packed
				 * hash tables . */

/* ================ Function decls ================ */

void    myerror();
void    readmap();
void    print_grid(), print_grid_histogram();
void    create_grid(), extract_edges(), classify_edges();

HASH_RECORD_T *create_hash_table(), *pack_hash_table();
float   find_hash_key(), find_hash_key_add_value();
int     get_hash_n_records();
int     calc_areas();
char   *malloc(), *calloc(), *realloc(), *malloc2();
double  read_delta_time();

/* ================ end ================ */
