/** Print.c  Print assorted data structures

Wm. Randolph Franklin

Last modified: by Wm Randolph Franklin (wrf) on mab.ecse.rpi.edu at Sun Nov 24 18:41:17 1991
	%W%	%G%	*/

#include "overlay.h"

/* Print the chains in one map */

print_chains(chain, n)
    CHAIN   chain[];
    int     n;

{
    int     i;
    printf("\n# Chains= %d. i, num_vert, lpol, rpol= \n", n);
    for (i = 0; i < n; i++)
	printf("   %d: %d %d %d %d\n", i, chain[i].base, chain[i].num_vert, chain[i].lpol, chain[i].rpol);
}

/* Print the vertices in one map, in both formats */

print_vertices(v, n, fvertex)
    VERTEX_T v[];
    int     n;
    FLOAT_VERTEX_T fvertex[];

{
    int     i;
    printf("\n# Vertices= %d. Internal: x,y. External: x,y\n", n);
    for (i = 0; i < n; i++)
	printf("   %3d: (%6d, %6d).  (%8.3f, %8.3f).\n", i, v[i].x, v[i].y, fvertex[i].x, fvertex[i].y);
}


print_output_areas(table)
    HASH_RECORD_T *table;
{
    int     i, n, k1, k2;
    HASH_KEY_T k;
    HASH_RECORD_T *list;
    float   this_area, tol;

    tol = rrx * rry * 200;	/* zero any smaller areas */
    n = get_hash_n_records(table);
    list = pack_hash_table(table);
    printf("\nAreas of %d output polygons in this map:\n", n);
    for (i = 0; i < n; i++) {
	k = (list + i)->k;
	k1 = INPUT_POLYGON_1(k);
	k2 = INPUT_POLYGON_2(k);
	/* The 0.5 is part of the area formula */
	this_area = (list + i)->v * 0.5;
	if ((-tol) < this_area && this_area < tol);
	else
	    printf(" %5d x%5d: %9.5g\n", k1, k2, this_area);
    }
}



print_int_array(title, a, n)	/* Print an int array with title and appropriate
				 * \ns. */
    char   *title;
    int    *a, n;
{
    int     i;
    printf(title);
    for (i = 0; i < n; i++) {
	if ((i % 20) == 0)
	    printf("\n");
	printf(" %3d", a[i]);
    }
    printf("\n");
}



print_short_int_array(title, a, n)	/* Print an int array with title and
					 * appropriate \ns. */
    char   *title;
    short int *a;
    int     n;
{
    int     i;
    printf(title);
    for (i = 0; i < n; i++) {
	if ((i % 20) == 0)
	    printf("\n");
	printf(" %3d", a[i]);
    }
    printf("\n");
}


print_ushort_int_array(title, a, n)	/* Print an unsigned short int array with
					 * title and appropriate \ns. */
    char   *title;
    unsigned short int *a;
    int     n;
{
    int     i;
    printf(title);
    for (i = 0; i < n; i++) {
	if ((i % 20) == 0)
	    printf("\n");
	printf(" %3d", a[i]);
    }
    printf("\n");
}
