/**		read_pgraph.c

Read one map.  Don't scale now; all the maps will be scaled together
later.  Also calc bounds.

%W%	%G%

Created by:	Venkatesh K. Sivaswami (Venki).
Modified by:    Wm. Randolph Franklin, 2/90
Function:	To read in the planar graph data

Output:		FLOAT_VERTEX_T fvertex []: Vertices
		CHAIN chain[]; Chains
		int *pnv: # verts
		int *pn_chains; Number of chains
		RANGE_T *xmin etc:  Map bounds.

Last modified: Tue Aug 21 16:21:04 EDT 1990
	
*/

#include  <stdio.h>
#include  <math.h>
#include "overlay.h"

read_pgraph(map_name, fvertex, pnv, chain, pn_chains, xmin, xmax, ymin, ymax)
    char    map_name[];
    FLOAT_VERTEX_T fvertex[];
    int    *pnv;
    CHAIN   chain[];
    int    *pn_chains;
    RANGE_T *xmin, *xmax, *ymin, *ymax;

{
    int     i, j, llp, rrp;
    int     temp_num_vert;	/* Number of vertices in each polygon chain */
    int     dum;
    int     hi_pts_per_chain;
    FILE   *fp_pgraph;		/* Pointer to planar graph data file */

    if ((fp_pgraph = fopen(map_name, "r")) == NULL) {
	fprintf(stderr, "File does not exist\n");
	exit(1);
    }
    i = 0;
    hi_pts_per_chain = 0;
    *pn_chains = 0;

    /* Read in the number of vertices  */

    while ((fscanf(fp_pgraph, "%d%d%d%d", &dum, &temp_num_vert, &dum, &dum)) != EOF) {
	/* Need an extra chain for last base. Hence the -1. */
	if (*pn_chains >= MAX_CHAINS - 1)
	    myerror("Too many input chains.");

	/* various criteria for determining the end of the file */

	if (temp_num_vert <= 0 || temp_num_vert == 999 || temp_num_vert == 9999)
	    break;
	if (temp_num_vert >= MAX_PTS_PER_CHAIN)
	    myerror("Too many points in this chain.");

	/*
	 * Read in the left and right polygons for each chain. Reading into an int
	 * and then assigning to chain.lpol means that if we change the type of
	 * lpol, we don't have to change the scan format.
	 */

	if (EOF == fscanf(fp_pgraph, "%d%d", &llp, &rrp)) {
	    perror("unexpectedly while reading chain header");
	    exit(1);
	};
	chain[*pn_chains].base = i;
	chain[*pn_chains].lpol = llp;
	chain[*pn_chains].rpol = rrp;

	/* Read in the coordinates of the vertices  */

	for (j = 0; j < temp_num_vert; j++) {

	    if (EOF == fscanf(fp_pgraph, "%f %f", &fvertex[i].x, &fvertex[i].y)) {
		perror("unexpectedly while reading chain");
		exit(1);
	    };
	    i++;

	}			/* for j */

	/* Store the number of vertices in polygon chain in array */

	chain[(*pn_chains)++].num_vert = temp_num_vert;
	if (hi_pts_per_chain < temp_num_vert)
	    hi_pts_per_chain = temp_num_vert;

    }

    chain[*pn_chains].base = *pnv = i;


    /* Find the minimum and maximum */

    *xmin = *xmax = fvertex[0].x;
    *ymin = *ymax = fvertex[0].y;

    for (i = 1; i < *pnv; i++) {

	if (fvertex[i].x < *xmin)
	    *xmin = fvertex[i].x;
	if (fvertex[i].x > *xmax)
	    *xmax = fvertex[i].x;
	if (fvertex[i].y < *ymin)
	    *ymin = fvertex[i].y;
	if (fvertex[i].y > *ymax)
	    *ymax = fvertex[i].y;
    }
    fclose(fp_pgraph);
    printf("Number of chains= %d\nGreatest number of points per chain= %d\n",
	   *pn_chains, hi_pts_per_chain);
}
