/** select_polygon.c  Select all the chains relating to two given polygons, one in each map,
    print them nicely, and optionally plot them (in a graphicstool window).

usage: Run in a graphics window.
       Type: select_polygon map1 pol-id1 map2 pol-id2 plot?  > output
       plot? = 0 if no plotting at all
               1 if just the lines
	       2 if also the chain endpoints labeled
	       3 if also every point labeled.

For 2 and 3, wait between chains, and plot successive chains in different line styles. 

It's ok to give a nonexistent polygon id if you want to show only one polygon.

Wm. Randolph Franklin

Last modified: by Wm Randolph Franklin (wrf) on mab.ecse.rpi.edu at Sun Nov 24 17:54:25 1991
	%W%	%G%
 */

#include  <stdio.h>
#include "overlay.h"
#include <usercore.h>
#include <suntool/sunview.h>
#include <suntool/panel.h>

int     pixwindd();
struct vwsurf vwsurf =
DEFAULT_VWSURF(pixwindd);

float   xmin, xmax, ymin, ymax;

#define MAXMAPS 2
#define MAXVERTS 1000		/* Values from overlay.h are too big */
#define MAXCHAINS 30

FLOAT_VERTEX_T fvertex[MAXMAPS][MAXVERTS];
CHAIN   chain[MAXMAPS][MAXCHAINS];
int     chain_id[MAXMAPS][MAXCHAINS];   /* should be part of chain */
int     this_chain_id;

main(argc, argv)
    int     argc;
    char  **argv;
{
    int     pol[MAXMAPS];
    float   px[MAX_PTS_PER_CHAIN], py[MAX_PTS_PER_CHAIN];
    int     nch[MAXMAPS], plotp;
    int     i, j, k, llp, rrp, nv, n, map;
    int     t;			/* Number of vertices in each polygon chain */
    int     dum, base;
    FILE   *(fin[MAXMAPS]);
    char    dummy;

    printf("argc= %d\n", argc);
    if (argc < 1 + 2 * MAXMAPS)
	myerror("Usage: select_polygon {<map> <polid>}*MAXMAPS <plot?>\n");

    if (EOF == sscanf(argv[1 + 2 * MAXMAPS], "%d", &plotp))
	perror("Can't get plotp");

    xmin = ymin = 1.0e30;
    xmax = ymax = -1.0e30;

    for (map = 0; map < MAXMAPS; map++) {

	printf("Processing map # %d\n", map);
	fin[map] = fopen(argv[1 + 2 * map], "r");
	if (fin[map] == NULL)
	    myerror("File doesn't exist.");

	if (EOF == sscanf(argv[2 + 2 * map], "%d", &pol[map]))
	    perror("Can't get polygon #");
	printf("Searching for polygon %d\n", pol[map]);


	i = 0;
	nch[map] = 0;

	/*
	 * Read in the number of vertices. Read in the left and right polygons for
	 * each chain. Reading into an int and then assigning to chain.lpol means
	 * that if we change the type of lpol, we don't have to change the scan
	 * format.
	 */

	while
	    ((fscanf(fin[map], "%d%d%d%d%d%d", &this_chain_id, &t, &dum, &dum, &llp, &rrp)) != EOF) {
	    if (t <= 0 || t >= MAX_PTS_PER_CHAIN)
		break;
	    chain[map][nch[map]].lpol = llp;
	    chain[map][nch[map]].rpol = rrp;
	    chain_id[map][nch[map]] = this_chain_id;

	    /* Read in the coordinates of the vertices  */

	    base = i;

	    for (j = 0; j < t; j++) {

		if (EOF == fscanf(fin[map], "%f %f", &fvertex[map][i].x, &fvertex[map][i].y)) {
		    perror("unexpectedly while reading chain");
		    exit(1);
		};
		i++;

	    }			/* for j */

	    /* Store the number of vertices in polygon chain in array */

	    chain[map][nch[map]].num_vert = t;

	    /* Keep this chain iff it borders the desired polygon. */

	    if (llp == pol[map] || rrp == pol[map])
		nch[map]++;
	    else
		i = base;

	}

	nv = 0;			/* Find the minimum and maximum */

	for (k = 0; k < nch[map]; k++) {

	    for (j = 0; j < chain[map][k].num_vert; j++) {

		if (fvertex[map][nv].x < xmin)
		    xmin = fvertex[map][nv].x;
		if (fvertex[map][nv].x > xmax)
		    xmax = fvertex[map][nv].x;
		if (fvertex[map][nv].y < ymin)
		    ymin = fvertex[map][nv].y;
		if (fvertex[map][nv].y > ymax)
		    ymax = fvertex[map][nv].y;
		nv++;
	    }
	}

	printf("The bounds of polygon %d, and selected polygons on earlier maps, are llc: (%f, %f), urc: (%f, %f)\n",
	       pol[map], xmin, ymin, xmax, ymax);

	/* Print the desired chains */

	printf("The %d chains bordering polygon %d are:\n", nch[map], pol[map]);

	base = 0;
	for (k = 0; k < nch[map]; k++) {
	    n = chain[map][k].num_vert;
	    printf("Chain #%d, id=%d, npts= %d, lpol= %d, rpol= %d\n", k, 
		   chain_id[map][k], n, chain[map][k].lpol, chain[map][k].rpol);
	    for (j = 0; j < n; j++)
		printf("   %3d: %8.3f %8.3f\n", j, fvertex[map][base + j].x,
		       fvertex[map][base + j].y);
	    base += n;
	}
    }

    if (plotp) {		/* Plot the polygons */

	start_graphics();

	for (map = 0; map < MAXMAPS; map++) {

	    set_linestyle(map);	/* SOLID, DOTTED, DASHED, DOTDASHED */
	    base = 0;
	    for (k = 0; k < nch[map]; k++) {
		n = chain[map][k].num_vert;
		sleep(2);
		for (j = 0; j < n; j++) {
		    px[j] = fvertex[map][base + j].x;
		    py[j] = fvertex[map][base + j].y;
		    if (j != 0 && plotp >= 3 && (n <= 50 || (j % 10) == 0)) {
			if (j < 5 || (n > 50 && j < 50))
			    sleep(1);
			draw_int(px[j], py[j], "%d", j);
		    }
		}
		/* marker_abs_2(px[0], py[0]); */
		if (plotp >= 2)
		    draw_int(px[0], py[0], "   Chain %d", k);
		move_abs_2(px[0], py[0]);
		polyline_abs_2(px, py, n);	/* If I started at 2nd point, then 1
						 * point chains would bomb here */
		base += n;
		if (plotp>=2) {
		  printf("Continue? ");
		  scanf("%c",&dummy);
		  set_linestyle(k&3);
	      }
	    }

	}
	end_graphics();
    }
}


start_graphics()
{
    float   xr, yr, xmin2, xmax2, ymin2, ymax2;
    float   eps = .1;

    initialize_core(BUFFERED, SYNCHRONOUS, THREED);
    initialize_view_surface(&vwsurf, FALSE);
    select_view_surface(&vwsurf);
    set_viewport_2(.01, .99, .01, .74);

    xr = xmax - xmin;
    yr = ymax - ymin;
    xmax2 = xmax + xr * eps;
    xmin2 = xmin - xr * eps;
    ymax2 = ymax + yr * eps;
    ymin2 = ymin - yr * eps;

    set_window(xmin2, xmax2, ymin2, ymax2);
    new_frame();
    set_marker_symbol('*');
    set_image_transformation_type(XFORM2);
    set_output_clipping(TRUE);
    create_retained_segment(1);
}


end_graphics()
{
    float   s, a, tx, ty, cx, cy;
    close_retained_segment();
    a = 0.0;
    cx = (xmax + xmin) / 2;
    cy = (ymax + ymin) / 2;
    for (;;) {
	printf("Scale factor, tx, ty? ");
	scanf("%f %f %f", &s, &tx, &ty);
	set_segment_image_transformation_2(1, s, s, a, tx, ty);
    }
    sleep(1000);		/* since terminate_core erases picture */
    deselect_view_surface(&vwsurf);
    terminate_core();
}


/* Draw a float in %8.3f at the given position */

draw_float(x, y, f)
    float   x, y, f;
{
    char    s[100];
    sprintf(s, "%8.3f", f);
    move_abs_2(x, y);
    text(s);
}

/* Draw an int in given format at the given position */

draw_int(x, y, format, i)
    float   x, y;
    char   *format;
    int     i;
{
    char    s[100];
    sprintf(s, format, i);
    move_abs_2(x, y);
    text(s);
}


/* Draw 2 ints in given format at the given position */

draw_2int(x, y, format, i, j)
    float   x, y;
    char   *format;
    int     i, j;
{
    char    s[100];
    sprintf(s, format, i, j);
    move_abs_2(x, y);
    text(s);
}
